// adapted original War Movie shader by Battlefront
// part of the code used from postprocessing shader (code by IÒigo Quilez)
// http://www.iquilezles.org/
// http://www.iquilezles.org/apps/shadertoy/index2.html
// (C)2010 IÒigo Quilez, implementation (C) 2011 Paul Houx
// Combat Mission adaptation and implementation (2015) by BarbaricCo 

#version 120
#extension GL_EXT_gpu_shader4 : enable

float luma_mod;
vec3 avg_luma_vec;

uniform float time;
uniform sampler2D tex0;

// For all settings: 1.0 = 100% 0.5=50% 1.5 = 150%
vec3 BrightnessSaturationContrast(vec3 color, float brt, float sat, float con)
{
	// Increase or decrease theese values to adjust r, g and b color channels seperately
	/*const*/ float AvgLumR = 0.5; //avg_luma_vec.r; //0.5;
	/*const*/ float AvgLumG = 0.5; //avg_luma_vec.g; //0.5;
	/*const*/ float AvgLumB = 0.45; //avg_luma_vec.b; //0.5;
	
	const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
	
	vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
	vec3 brtColor = color * brt;
	vec3 intensity = vec3(dot(brtColor, LumCoeff));
	vec3 satColor = mix(intensity, brtColor, sat);
	//vec3 conColor = mix((intensity - AvgLumin) / AvgLumin, satColor, con); 
	
	vec3 conColor = mix(mix(AvgLumin, intensity, 0.5) /*intensity*/, satColor, con);
	return conColor;
}

// bleach bypass
vec3 overlay(vec3 previousmix, vec3 amount)
{
    const vec3 lumcoeff = vec3(0.2125, 0.7154, 0.0721);
    const vec3 one = vec3(1.0);	
    const vec3 two = vec3(2.0);
    vec3 myInput = vec3(dot(previousmix, lumcoeff));
    
	float luminance = dot(previousmix,lumcoeff);
	float mixamount = clamp((luminance - 0.45) * 10.0, 0.0, 1.0);

	vec3 branch1 = two * previousmix * myInput;
	vec3 branch2 = one - (two * (one - previousmix) * (one - myInput));
	
	vec3 result = mix(branch1, branch2, vec3(mixamount));

	return mix(previousmix, result, amount);
}

vec4 bypass(vec4 base)
{    
    vec3 lumCoeff = vec3(0.35,0.35,0.1); //vec3(0.25,0.65,0.1);
    float lum = dot(lumCoeff,base.rgb);
    vec3 blend = vec3(lum);
    float L = min(1,max(0,10.0*(lum- 0.45)));
    vec3 result1 = 2.0f * base.rgb * blend; 
    vec3 result2 = 1.0f - 2.0f*(1.0f-blend)*(1.0f-base.rgb);
    vec3 newColor = mix(result1,result2,L); // was lerp
    float A2 = /*Opacity*/ 0.25 * base.a; //0.45 DEFAULT
    vec3 mixRGB = A2 * newColor.rgb; 
    mixRGB += ((1.0f-A2) * base.rgb); 
    return vec4(mixRGB,base.a);
}

void main(void)
{
	// initialize coordinates and colors
    vec2 q = gl_TexCoord[0].xy;
    vec2 uv = q;
    vec3 oricol = texture2D(tex0,vec2(q.x,q.y)).xyz;
    vec3 col;
	
	// zooming (disabled)
    //uv = 0.5 + (q-0.5)*(0.9 + 0.1*sin(0.2*time));

	// color separation
    col.r = texture2D(tex0,vec2(uv.x+0.00001,uv.y)).x;
    col.g = texture2D(tex0,vec2(uv.x+0.000,uv.y)).y;
    col.b = texture2D(tex0,vec2(uv.x-0.00001,uv.y)).z;

	// increase contrast
	col = clamp(col*1.0+0.9*col*col*0.45,0.0,1.0);

	// original
	// col = clamp(col*0.5+0.5*col*col*1.2,0.0,1.0);

	// vignetting
    col *= 1.12 + 0.80*5.0*uv.x*uv.y*(1.0-uv.x)*(1.0-uv.y);
	// vignetting default
    //col *= 0.6 + 0.5*16.0*uv.x*uv.y*(1.0-uv.x)*(1.0-uv.y);

	// tinting
    col *= vec3(0.95,0.94,0.95);

	// tv lines
    //col *= 0.9+0.1*sin(10.0*time+uv.y*1800.0);
	
	// default
	//col *= 0.9+0.1*sin(10.0*time+uv.y*1800.0);

	// flickering
    col *= 0.97+0.03*sin(110.0*time);
	
	//overlay
	col = overlay(col, vec3(0.3));

	// show/hide original (disabled)
    //float comp = smoothstep( 0.2, 0.7, sin(time) );
    //col = mix( col, oricol, clamp(-2.0+2.0*q.x+3.0*comp,0.0,1.0) );
	
	// set final color
    gl_FragColor = vec4(col,1.0);
	
    // a bypass filter (sort of Black Hawk Down-ish) that gives a nice effect
    gl_FragColor = bypass(gl_FragColor);
    gl_FragColor.xyz = BrightnessSaturationContrast(gl_FragColor.xyz, 0.93, 1.08, 1.05);
    
    gl_FragColor = mix(gl_FragColor, vec4(0.0, 0.0, 0.5, 1.0), 0.1);
    gl_FragColor = mix(gl_FragColor, vec4(139.0, 69.0, 19.0, 255.0) / 255.0, 0.07);
	
    // final brightness
    gl_FragColor *= 1.05;
}